/************************************************************************\
 FILE ..........: display.c
 AUTHOR ........: Vitaly Puzrin
 DESCRIPTION ...:      
 NOTES .........:
 COPYRIGHT .....: Vitaly Puzrin, 1998
 HISTORY .......: DATE        COMMENT
                  ---------------------------------------------------
                  25.06.1999
\************************************************************************/
#include    "common.h"
#include    "display.h"
#include    "time.h"
#include    "rpm.h"

#define    SDA_MASK     0x10    //     
#define    SCL_MASK     0x40    //     

void LcdBusWait(void);          //    LCD
void DisplayCh(BYTE byChar);    //  1   
void ShowDisplayData(void);     //   
void PrepareRpmData(WORD wRpm, BYTE byBladeCnt);
void PrepareTimeData(WORD wTime, BOOLEAN btWorkFlag);

BOOLEAN btDisplayUpdate;        // = TRUE,     

BOOLEAN btRpmMode;              //   :
                                // TRUE -    
                                // FALSE -  

BYTE byDisplay[10];        //  ,    

//  
flash BYTE byCharacter[16] = {
    0x0A, 0x01, 0x02, 0x03,
    0x04, 0x05, 0x06, 0x07,
    0x08, 0x09, 0x00, 
                      0x0B,
    0x0C, 0x0D, 0x0E, 0x0F};

#define     chr_SPACE   10          //  
#define     chr_MINUS   15
#define     chr_F       11
#define     chr_BL      12
#define     chr_L       13
#define     chr_P       14    


/************************************************************************\
   .
      :  -
      : -
\************************************************************************/
void RefreshDisplay(void)
{
    WORD    wLockedRpm, wLockedTime;
    
    __disable_interrupt();
    wLockedRpm = wRpm;
    wLockedTime = wTime;
    __enable_interrupt();
    
    if( btRpmMode )
        PrepareRpmData(wLockedRpm, byBladeCnt);
    else
        PrepareTimeData(wLockedTime, btTimerOn);
    ShowDisplayData();
}

/************************************************************************\
       
     
      :  wRpm -  , byBladeCnt -  
      : -
\************************************************************************/
void PrepareRpmData(WORD wRpm, BYTE byBladeCnt)
{
    BYTE i;

    //  5  -  
    for(i=0; i<5; i++)
    {
        byDisplay[4-i] = wRpm % 10;
        wRpm /= 10;
    }

    //  4  -  
    byDisplay[5] = chr_SPACE;
    byDisplay[6] = chr_SPACE;
    byDisplay[7] = chr_SPACE;
    byDisplay[8] = chr_SPACE;

    //   -    
    byDisplay[9] = byBladeCnt;
}

/************************************************************************\
    
     
      :  ulTime - , btWorkFlag - /
      : -
\************************************************************************/
void PrepareTimeData(WORD wTime, BOOLEAN btWorkFlag)
{
    WORD i;

    //    ( 
    //    )
    if (btWorkFlag)
        byDisplay[0] = chr_BL;
    else
        byDisplay[0] = chr_SPACE;
        
    byDisplay[1] = chr_SPACE;
    // 
    i = wTime / 60;
    byDisplay[2] = i / 10;
    byDisplay[3] = i % 10;
    byDisplay[4] = chr_MINUS;
    
    // 
    i = (wTime % 60);
    byDisplay[5] = i / 10;
    byDisplay[6] = i % 10;

    byDisplay[7] = chr_SPACE;
    byDisplay[8] = chr_SPACE;
    byDisplay[9] = chr_SPACE;
}

/************************************************************************\
      .
      :  -
      : -
\************************************************************************/
void ShowDisplayData(void)
{
    BYTE byChCnt;    //   .
    
    for(byChCnt=0; byChCnt<10; byChCnt++)
        DisplayCh(byCharacter[byDisplay[byChCnt]]);
}

/************************************************************************\
   1   
      :  byChar -  
      : -
\************************************************************************/
void DisplayCh(BYTE byChar)
{
    BYTE byBitCnt;    //     .

    for(byBitCnt=0; byBitCnt<4; byBitCnt++)
    {
        if( byChar & 0x08 )
            PORTB |= SDA_MASK;
        else
            PORTB &= ~SDA_MASK;
        LcdBusWait();
        PORTB &= ~SCL_MASK;
        LcdBusWait();
        PORTB |= SCL_MASK;
        byChar <<= 1;                 
    }
}

/************************************************************************\
     
      :  -
      : -
\************************************************************************/
void DisplayInit(void)
{
    BYTE byChCnt;

    btRpmMode = TRUE;       //    
                            //   .
    PORTB |= SDA_MASK;
    PORTB |= SCL_MASK;
    LcdBusWait();
    
    for(byChCnt=0; byChCnt<10; byChCnt++)
        byDisplay[byChCnt] = chr_SPACE;
    ShowDisplayData();

}

/************************************************************************\
     LCD (     )
      :  -
      : -
\************************************************************************/
void LcdBusWait(void)
{
    BYTE i;
    
    for(i=0; i<50; i++);
}
